<?php
/*
Plugin Name: post-views
Plugin URI: http://ziming.org/dev/post-views
Description: Record how many times the post have been views and show it. 
Version: 1.2.1
Author: Suny Tse
Author URI: http://ziming.org
*/

/*  Copyright 2010  Suny Tse  (email : message@ziming.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

add_action('init', 'postviews_textdomain');
add_action('admin_menu', 'postviews_menu');
add_action('wp_head', 'process_postviews');
add_action('widgets_init', 'widget_views_init');
add_action('publish_post', 'add_views_fields');
add_action('publish_page', 'add_views_fields');
add_filter('query_vars', 'views_variables');
add_action('pre_get_posts', 'views_sorting');


### Create Text Domain For Translations

function postviews_textdomain() {
	load_plugin_textdomain('post-views', false, 'post-views');
}

### Add Option Menu

function postviews_menu() {
	if (function_exists('add_options_page')) {
		add_options_page(__('Post-Views', 'post-views'), __('Post-Views', 'post-views'), 'manage_options', 'post-views/postviews-uninstall.php') ;
	}
}

### Init WP-PostViews Widget

function widget_views_init() {
	register_widget('WP_Widget_PostViews');
}

if(!function_exists(is_bot)){
	function is_bot($agent) {
		
		$is_bot = false;		
		if ($agent ==''){
			$is_bot = true;
		}
		if (strlen($agent)<50){
			$is_bot = true;
		}
		$bots = array('Google Bot' => 'googlebot', 'Google Bot' => 'google', 'MSN' => 'msnbot', 'Alex' => 'ia_archiver', 'Lycos' => 'lycos', 'Ask Jeeves' => 'jeeves', 'Altavista' => 'scooter', 'AllTheWeb' => 'fast-webcrawler', 'Inktomi' => 'slurp@inktomi', 'Turnitin.com' => 'turnitinbot', 'Technorati' => 'technorati', 'Yahoo' => 'yahoo', 'Findexa' => 'findexa', 'NextLinks' => 'findlinks', 'Gais' => 'gaisbo', 'WiseNut' => 'zyborg', 'WhoisSource' => 'surveybot', 'Bloglines' => 'bloglines', 'BlogSearch' => 'blogsearch', 'PubSub' => 'pubsub', 'Syndic8' => 'syndic8', 'RadioUserland' => 'userland', 'Gigabot' => 'gigabot', 'Become.com' => 'become.com');
		foreach ($bots as $name => $lookfor) { 
			if (stristr($useragent, $lookfor) !== false) { 
					$is_bot = true;
					break;
			}
		}
		return $is_bot;
	}
}

function process_postviews() {
	global $user_ID, $post;
	if(!wp_is_post_revision($post)) {
			if(is_single() || is_page()) {
					$id = intval($post->ID);
					$post_views = get_post_custom($id);
					$post_views = intval($post_views['views'][0]);
					$post_bot_views = intval($post_views['bot_views'][0]);
					$is_bot=false;

					$useragent = $_SERVER['HTTP_USER_AGENT'];
					if(is_bot($useragent)){
							if(!update_post_meta($id, 'bot_views', ($post_bot_views+1))) {
									add_post_meta($id, 'bot_views', 1, true);
							}				
					}else{
							if(!update_post_meta($id, 'views', ($post_views+1))) {
									add_post_meta($id, 'views', 1, true);
							}
					}
			}
	}
}

### Function: Display The Post Views

function get_views($type = 'normal') {
	$post_views = intval(post_custom('views'));
	$post_bot_views = intval(post_custom('bot_views'));
	if ($type == 'normal') {
			echo apply_filters('the_views', $post_views);
	}else if($type == 'bot'){
			echo apply_filters('the_bot_views', $post_bot_views);		
	}
}


### Function: Display Total Views

if(!function_exists('get_totalviews')) {
	function get_totalviews($type = 'normal') {
		global $wpdb;
		if($type == 'normal') {
			$total_views = intval($wpdb->get_var("SELECT SUM(meta_value+0) FROM $wpdb->postmeta WHERE meta_key = 'views'"));
			echo $total_views;
		} else if($type = 'bot') {
			$total_bot_views = intval($wpdb->get_var("SELECT SUM(meta_value+0) FROM $wpdb->postmeta WHERE meta_key = 'bot_views'"));
			return $total_bot_views;
		}
	}
}


### Function: Snippet Text
if(!function_exists('snippet_text')) {
	function snippet_text($text, $length = 0) {
		if (defined('MB_OVERLOAD_STRING')) {
		  $text = @html_entity_decode($text, ENT_QUOTES, get_option('blog_charset'));
		 	if (mb_strlen($text) > $length) {
				return htmlentities(mb_substr($text,0,$length), ENT_COMPAT, get_option('blog_charset')).'...';
		 	} else {
				return htmlentities($text, ENT_COMPAT, get_option('blog_charset'));
		 	}
		} else {
			$text = @html_entity_decode($text, ENT_QUOTES, get_option('blog_charset'));
		 	if (strlen($text) > $length) {
				return htmlentities(substr($text,0,$length), ENT_COMPAT, get_option('blog_charset')).'...';
		 	} else {
				return htmlentities($text, ENT_COMPAT, get_option('blog_charset'));
		 	}
		}
	}
}


### Function: Modify Default WordPress Listing To Make It Sorted By Post Views

function views_fields($content) {
	global $wpdb;
	$content .= ", ($wpdb->postmeta.meta_value+0) AS views";
	return $content;
}
function views_join($content) {
	global $wpdb;
	$content .= " LEFT JOIN $wpdb->postmeta ON $wpdb->postmeta.post_id = $wpdb->posts.ID";
	return $content;
}
function views_where($content) {
	global $wpdb;
	$content .= " AND $wpdb->postmeta.meta_key = 'views'";
	return $content;
}
function views_orderby($content) {
	$orderby = trim(addslashes(get_query_var('v_orderby')));
	if(empty($orderby) || ($orderby != 'asc' && $orderby != 'desc')) {
		$orderby = 'desc';
	}
	$content = " views $orderby";
	return $content;
}


### Function: Add Views Custom Fields

function add_views_fields($post_ID) {
	global $wpdb;
	if(!wp_is_post_revision($post_ID)) {
		add_post_meta($post_ID, 'views', 0, true);
		add_post_meta($post_ID, 'bot_views', 0, true);
	}
}


### Function: Delete Views Custom Fields
add_action('delete_post', 'delete_views_fields');
function delete_views_fields($post_ID) {
	global $wpdb;
	if(!wp_is_post_revision($post_ID)) {
		delete_post_meta($post_ID, 'views');
		delete_post_meta($post_ID, 'bot_views');
	}
}


### Function: Public Variables

function views_variables($public_query_vars) {
	$public_query_vars[] = 'v_sortby';
	$public_query_vars[] = 'v_orderby';
	return $public_query_vars;
}


### Function: Sort Views Posts

function views_sorting($local_wp_query) {
	if($local_wp_query->get('v_sortby') == 'views') {
		add_filter('posts_fields', 'views_fields');
		add_filter('posts_join', 'views_join');
		add_filter('posts_where', 'views_where');
		add_filter('posts_orderby', 'views_orderby');
	} else {
		remove_filter('posts_fields', 'views_fields');
		remove_filter('posts_join', 'views_join');
		remove_filter('posts_where', 'views_where');
		remove_filter('posts_orderby', 'views_orderby');
	}
}
/*
### Function: Increment Post Views
increment_views();

function increment_views() {
	global $wpdb;
	$post_id = intval($_GET['postviews_id']);
	if($post_id > 0) {
		$post_views = get_post_custom($post_id);
		$post_views = intval($post_views['views'][0]);
		$post_bot_views = intval($post_views['views'][0]);
		if(!update_post_meta($post_id, 'views', ($post_views+1))) {
			add_post_meta($post_id, 'views', 1, true);
		}
		if(!update_post_meta($post_id, 'bot_views', ($post_views+1))) {
			add_post_meta($post_id, 'bot_views', 1, true);
		}
	}
}
*/


class WP_Widget_PostViews extends WP_Widget {
	// Constructor
	function WP_Widget_PostViews() {
		$widget_ops = array('description' => __('Post Views', 'post-views'));
		$this->WP_Widget('views', __('Views', 'post-views'), $widget_ops);
	}

	// Display Widget
	function widget($args, $instance) {
		extract($args);
		$title = apply_filters('widget_title', esc_attr($instance['title']));
		$type = esc_attr($instance['type']);
		$mode = esc_attr($instance['mode']);
		$limit = intval($instance['limit']);
		$chars = intval($instance['chars']);
		if(esc_attr($instance['cat_ids'])==0){
				$cat_ids =	0;
		}else{
				$cat_ids = explode(',', esc_attr($instance['cat_ids']));
		}
		$tag_ids = $cat_ids;
		echo $before_widget.$before_title.$title.$after_title;
		echo '<ul>'."\n";
		switch($type) {
			case 'least_viewed':
				get_least_viewed($mode, $limit, $chars);
				break;
			case 'most_viewed':
				get_most_viewed($mode, $limit, $chars);
				break;
			case 'most_viewed_category':
				get_most_viewed_category($cat_ids, $mode, $limit, $chars);
				break;
			case 'least_viewed_category':
				get_least_viewed_category($cat_ids, $mode, $limit, $chars);
				break;
				case 'most_viewed_tag':
				get_most_viewed_tag($tag_ids, $mode, $limit, $chars);
				break;
			case 'least_viewed_tag':
				get_least_viewed_tag($tag_ids, $mode, $limit, $chars);
				break;
		}
		echo '</ul>'."\n";
		echo $after_widget;
	}

	// When Widget Control Form Is Posted
	function update($new_instance, $old_instance) {
		if (!isset($new_instance['submit'])) {
			return false;
		}
		$instance = $old_instance;
		$instance['title'] = strip_tags($new_instance['title']);
		$instance['type'] = strip_tags($new_instance['type']);
		$instance['mode'] = strip_tags($new_instance['mode']);
		$instance['limit'] = intval($new_instance['limit']);
		$instance['chars'] = intval($new_instance['chars']);
		$instance['cat_ids'] = strip_tags($new_instance['cat_ids']);
		return $instance;
	}

	// DIsplay Widget Control Form
	function form($instance) {
		global $wpdb;
		$instance = wp_parse_args((array) $instance, array('title' => __('Views', 'post-views'), 'type' => 'most_viewed', 'mode' => 'both', 'limit' => 10, 'chars' => 200, 'cat_ids' => '0'));
		$title = esc_attr($instance['title']);
		$type = esc_attr($instance['type']);
		$mode = esc_attr($instance['mode']);
		$limit = intval($instance['limit']);
		$chars = intval($instance['chars']);
		$cat_ids = esc_attr($instance['cat_ids']);
?>
		<p>
			<label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:', 'post-views'); ?> <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo $title; ?>" /></label>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('type'); ?>"><?php _e('Statistics Type:', 'post-views'); ?>
				<select name="<?php echo $this->get_field_name('type'); ?>" id="<?php echo $this->get_field_id('type'); ?>" class="widefat">
					<option value="least_viewed"<?php selected('least_viewed', $type); ?>><?php _e('Least Viewed', 'post-views'); ?></option>
					<option value="least_viewed_category"<?php selected('least_viewed_category', $type); ?>><?php _e('Least Viewed By Category', 'post-views'); ?></option>
					<optgroup>--------------</optgroup>
					<option value="most_viewed"<?php selected('most_viewed', $type); ?>><?php _e('Most Viewed', 'post-views'); ?></option>
					<option value="most_viewed_category"<?php selected('most_viewed_category', $type); ?>><?php _e('Most Viewed By Category', 'post-views'); ?></option>
					<optgroup>--------------</optgroup>
					<option value="least_viewed_tag"<?php selected('least_viewed_tag', $type); ?>><?php _e('Least Viewed By Tag', 'post-views'); ?></option>
					<option value="most_viewed_tag"<?php selected('most_viewed_tag', $type); ?>><?php _e('Most Viewed By Tag', 'post-views'); ?></option>
				</select>
			</label>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('mode'); ?>"><?php _e('Include Views From:', 'post-views'); ?>
				<select name="<?php echo $this->get_field_name('mode'); ?>" id="<?php echo $this->get_field_id('mode'); ?>" class="widefat">
					<option value="both"<?php selected('both', $mode); ?>><?php _e('Posts &amp; Pages', 'post-views'); ?></option>
					<option value="post"<?php selected('post', $mode); ?>><?php _e('Posts Only', 'post-views'); ?></option>
					<option value="page"<?php selected('page', $mode); ?>><?php _e('Pages Only', 'post-views'); ?></option>
				</select>
			</label>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('limit'); ?>"><?php _e('No. Of Records To Show:', 'post-views'); ?> <input class="widefat" id="<?php echo $this->get_field_id('limit'); ?>" name="<?php echo $this->get_field_name('limit'); ?>" type="text" value="<?php echo $limit; ?>" /></label>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('chars'); ?>"><?php _e('Maximum Post Title Length (Characters):', 'post-views'); ?> <input class="widefat" id="<?php echo $this->get_field_id('chars'); ?>" name="<?php echo $this->get_field_name('chars'); ?>" type="text" value="<?php echo $chars; ?>" /></label><br />
			<small><?php _e('<strong>0</strong> to disable.', 'post-views'); ?></small>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('cat_ids'); ?>"><?php _e('Category/Tag IDs:', 'post-views'); ?> <span style="color: red;">*</span> <input class="widefat" id="<?php echo $this->get_field_id('cat_ids'); ?>" name="<?php echo $this->get_field_name('cat_ids'); ?>" type="text" value="<?php echo $cat_ids; ?>" /></label><br />
			<small><?php _e('Seperate mutiple categories with commas.', 'post-views'); ?></small>
		</p>
		<p style="color: red;">
			<small><?php _e('* If you are not using any category statistics, you can ignore it.', 'post-views'); ?></small>
		<p>
		<input type="hidden" id="<?php echo $this->get_field_id('submit'); ?>" name="<?php echo $this->get_field_name('submit'); ?>" value="1" />
<?php
	}
}

###Display Least Viewed Page/Post

if(!function_exists('get_least_viewed')) {
	function get_least_viewed($mode = '', $limit = 10, $chars = 0, $display = true) {
		global $wpdb, $post;
		$output = '';
		$where = '';
		if(!empty($mode) && $mode != 'both') {
			$where = "post_type = '$mode'";
		} else {
			$where = '1=1';
		}
		$most_viewed = $wpdb->get_results("SELECT DISTINCT $wpdb->posts.*, (meta_value+0) AS views FROM $wpdb->posts LEFT JOIN $wpdb->postmeta ON $wpdb->postmeta.post_id = $wpdb->posts.ID WHERE post_date < '".current_time('mysql')."' AND post_status = 'publish' AND meta_key = 'views' AND $where AND post_password = '' ORDER  BY views ASC LIMIT $limit");
		if($most_viewed) {
			foreach ($most_viewed as $post) {
				$post_views = intval($post->views);
				$post_title = get_the_title();
				if($chars > 0) {
						if(!function_exists('cut_str')){
								$post_title = snippet_text($post_title, $chars);
						}else{
								$post_title = cut_str($post_title, $chars);
						}
				}
				$output .= '<li> <a href="'.get_permalink().'" class="with_bottom_a" >'.$post_title.'</a></li>'."\n";
			}			
		} else {
			$output = '<li>'.__('N/A', 'post-views').'</li>'."\n";
		}
		if($display) {
			echo $output;
		} else {
			return $output;
		}
	}
}


### Function: Display Most Viewed Page/Post
if(!function_exists('get_most_viewed')) {
	function get_most_viewed($mode = '', $limit = 10, $chars = 0, $display = true) {
		global $wpdb, $post;
		$output = '';
		$where = '';
		if(!empty($mode) && $mode != 'both') {
			$where = "post_type = '$mode'";
		}else {
			$where = '1=1';
		}
		$sql="SELECT DISTINCT $wpdb->posts.*, (meta_value+0) AS views FROM $wpdb->posts LEFT JOIN $wpdb->postmeta ON $wpdb->postmeta.post_id = $wpdb->posts.ID WHERE post_date < '".current_time('mysql')."' AND post_status = 'publish' AND meta_key = 'views' AND $where AND post_password = '' ORDER  BY views DESC LIMIT $limit";
		$most_viewed = $wpdb->get_results($sql);
		if($most_viewed) {
			foreach ($most_viewed as $post) {
				$post_views = intval($post->views);
				$post_title = get_the_title();
				if($chars > 0) {
						if(!function_exists('cut_str')){
								$post_title = snippet_text($post_title, $chars);
						}else{
								$post_title = cut_str($post_title, $chars);
						}
				}
				$output .= '<li> <a href="'.get_permalink().'" class="with_bottom_a" >'.$post_title.'</a></li>'."\n";
			}			
		} else {
			$output=$sql;
			//$output = '<li>'.__('N/A', 'post-views').'</li>'."\n";
		}
		if($display) {
			echo $output;
		} else {
			return $output;
		}
	}
}


### Function: Display Leased Viewed Page/Post By Category ID
if(!function_exists('get_least_viewed_category')) {
	function get_least_viewed_category($category_id = 0, $mode = '', $limit = 10, $chars = 0, $display = true) {
		global $wpdb, $post;
		$output = '';
		$where = '';
		if(!empty($mode) && $mode != 'both') {
			$where = "post_type = '$mode'";
		} else {
			$where = '1=1';
		}
		if(is_array($category_id)) {
			$category_sql = "$wpdb->term_taxonomy.term_id IN (".join(',', $category_id).')';
		}else if($category_id == 0){
			$category_sql = "1=1";
		} else {
			$category_sql = "$wpdb->term_taxonomy.term_id = $category_id";
		}
		$most_viewed = $wpdb->get_results("SELECT DISTINCT $wpdb->posts.*, (meta_value+0) AS views FROM $wpdb->posts LEFT JOIN $wpdb->postmeta ON $wpdb->postmeta.post_id = $wpdb->posts.ID INNER JOIN $wpdb->term_relationships ON ($wpdb->posts.ID = $wpdb->term_relationships.object_id) INNER JOIN $wpdb->term_taxonomy ON ($wpdb->term_relationships.term_taxonomy_id = $wpdb->term_taxonomy.term_taxonomy_id) WHERE post_date < '".current_time('mysql')."' AND $wpdb->term_taxonomy.taxonomy = 'category' AND $category_sql  AND post_status = 'publish' AND meta_key = 'views' AND $where AND post_password = '' ORDER  BY views ASC LIMIT $limit");
		if($most_viewed) {
			foreach ($most_viewed as $post) {
				$post_views = intval($post->views);
				$post_title = get_the_title();
				if($chars > 0) {
						if(!function_exists('cut_str')){
								$post_title = snippet_text($post_title, $chars);
						}else{
								$post_title = cut_str($post_title, $chars);
						}
				}
				$output .= '<li> <a href="'.get_permalink().'" class="with_bottom_a" >'.$post_title.'</a></li>'."\n";
			}
		} else {
			$output = '<li>'.__('N/A', 'wp-postviews').'</li>'."\n";
		}
		if($display) {
			echo $output;
		} else {
			return $output;
		}
	}
}


### Function: Display Most Viewed Page/Post By Category ID
if(!function_exists('get_most_viewed_category')) {
	function get_most_viewed_category($category_id = 0, $mode = '', $limit = 10, $chars = 0, $display = true) {
		global $wpdb, $post;
		$output = '';
		if(is_array($category_id)) {
			$category_sql = "$wpdb->term_taxonomy.term_id IN (".join(',', $category_id).')';
		}else if($category_id == 0){
			$category_id = '1=1';
		}else {
			$category_sql = "$wpdb->term_taxonomy.term_id = $category_id";
		}
		if(!empty($mode) && $mode != 'both') {
			$where = "post_type = '$mode'";
		} else {
			$where = '1=1';
		}
		$most_viewed = $wpdb->get_results("SELECT DISTINCT $wpdb->posts.*, (meta_value+0) AS views FROM $wpdb->posts LEFT JOIN $wpdb->postmeta ON $wpdb->postmeta.post_id = $wpdb->posts.ID INNER JOIN $wpdb->term_relationships ON ($wpdb->posts.ID = $wpdb->term_relationships.object_id) INNER JOIN $wpdb->term_taxonomy ON ($wpdb->term_relationships.term_taxonomy_id = $wpdb->term_taxonomy.term_taxonomy_id) WHERE post_date < '".current_time('mysql')."' AND $wpdb->term_taxonomy.taxonomy = 'category' AND $category_sql  AND post_status = 'publish' AND meta_key = 'views' AND $where AND post_password = '' ORDER  BY views DESC LIMIT $limit");
		if($most_viewed) {
			foreach ($most_viewed as $post) {
				$post_views = intval($post->views);
				$post_title = get_the_title();
				if($chars > 0) {
						if(!function_exists('cut_str')){
								$post_title = snippet_text($post_title, $chars);
						}else{
								$post_title = cut_str($post_title, $chars);
						}
				}
				$output .= '<li> <a href="'.get_permalink().'" class="with_bottom_a" >'.$post_title.'</a></li>'."\n";
			}
		} else {
			$output = '<li>'.__('N/A', 'wp-postviews').'</li>'."\n";
		}
		if($display) {
			echo $output;
		} else {
			return $output;
		}
	}
}


### Function: Display Most Viewed Page/Post By Tag ID
if(!function_exists('get_most_viewed_tag')) {
	function get_most_viewed_tag($tag_id = 0, $mode = '', $limit = 10, $chars = 0, $display = true) {
		global $wpdb, $post;
		$output = '';
		$where = '';
		if(!empty($mode) && $mode != 'both') {
			$where = "post_type = '$mode'";
		} else {
			$where = '1=1';
		}
		if(is_array($tag_id)) {
			$tag_sql = "$wpdb->term_taxonomy.term_id IN (".join(',', $tag_id).')';
		}else if($tag_id==0){
			$tag_sql = '1=1';
		}else {
			$tag_sql = "$wpdb->term_taxonomy.term_id = $tag_id";
		}
		$most_viewed = $wpdb->get_results("SELECT DISTINCT $wpdb->posts.*, (meta_value+0) AS views FROM $wpdb->posts LEFT JOIN $wpdb->postmeta ON $wpdb->postmeta.post_id = $wpdb->posts.ID INNER JOIN $wpdb->term_relationships ON ($wpdb->posts.ID = $wpdb->term_relationships.object_id) INNER JOIN $wpdb->term_taxonomy ON ($wpdb->term_relationships.term_taxonomy_id = $wpdb->term_taxonomy.term_taxonomy_id) WHERE post_date < '".current_time('mysql')."' AND $wpdb->term_taxonomy.taxonomy = 'post_tag' AND $tag_sql  AND post_status = 'publish' AND meta_key = 'views' AND $where AND post_password = '' ORDER  BY views DESC LIMIT $limit");
		if($most_viewed) {
			foreach ($most_viewed as $post) {
				$post_views = intval($post->views);
				$post_title = get_the_title();
				if($chars > 0) {
						if(!function_exists('cut_str')){
								$post_title = snippet_text($post_title, $chars);
						}else{
								$post_title = cut_str($post_title, $chars);
						}
				}
				$output .= '<li> <a href="'.get_permalink().'" class="with_bottom_a" >'.$post_title.'</a></li>'."\n";
			}
		} else {
			$output = '<li>'.__('N/A', 'wp-postviews').'</li>'."\n";
		}
		if($display) {
			echo $output;
		} else {
			return $output;
		}
	}
}


### Function: Display Least Viewed Page/Post By Tag ID
if(!function_exists('get_least_viewed_tag')) {
	function get_least_viewed_tag($tag_id = 0, $mode = '', $limit = 10, $chars = 0, $display = true) {
		global $wpdb, $post;
		$output = '';
		$where = '';
		if(!empty($mode) && $mode != 'both') {
			$where = "post_type = '$mode'";
		} else {
			$where = '1=1';
		}
		if(is_array($tag_id)) {
			$tag_sql = "$wpdb->term_taxonomy.term_id IN (".join(',', $tag_id).')';
		} else {
			$tag_sql = "$wpdb->term_taxonomy.term_id = $tag_id";
		}
		$most_viewed = $wpdb->get_results("SELECT DISTINCT $wpdb->posts.*, (meta_value+0) AS views FROM $wpdb->posts LEFT JOIN $wpdb->postmeta ON $wpdb->postmeta.post_id = $wpdb->posts.ID INNER JOIN $wpdb->term_relationships ON ($wpdb->posts.ID = $wpdb->term_relationships.object_id) INNER JOIN $wpdb->term_taxonomy ON ($wpdb->term_relationships.term_taxonomy_id = $wpdb->term_taxonomy.term_taxonomy_id) WHERE post_date < '".current_time('mysql')."' AND $wpdb->term_taxonomy.taxonomy = 'post_tag' AND $tag_sql AND post_status = 'publish' AND meta_key = 'views' AND $where AND post_password = '' ORDER  BY views ASC LIMIT $limit");
		if($most_viewed) {
			foreach ($most_viewed as $post) {
				$post_views = intval($post->views);
				$post_title = get_the_title();
				if($chars > 0) {
						if(!function_exists('cut_str')){
								$post_title = snippet_text($post_title, $chars);
						}else{
								$post_title = cut_str($post_title, $chars);
						}
				}
				$output .= '<li> <a href="'.get_permalink().'" class="with_bottom_a" >'.$post_title.'</a></li>'."\n";
			}
		} else {
			$output = '<li>'.__('N/A', 'wp-postviews').'</li>'."\n";
		}
		if($display) {
			echo $output;
		} else {
			return $output;
		}
	}
}
?>